from typing import TYPE_CHECKING
from ..constants import TextType
from . import fmts
from .basic import to_plain_name

if TYPE_CHECKING:
    from .. import ToolDelta


def get_playername_and_msg_from_text_packet(
    frame: "ToolDelta", pkt: dict
) -> tuple[str, str, bool] | tuple[None, None, bool]:
    """
    将 Text 数据包转换为玩家名与消息

    Args:
        frame (ToolDelta): ToolDelta 框架
        pkt (dict): 数据包

    Returns:
        tuple[str, str, bool] | tuple[None, None, bool]: 玩家名, 消息, 是否确认为玩家消息
    """
    msg: str = pkt["Message"]
    sender_name = ""

    if (extraData := pkt["NeteaseExtraData"]) and len(extraData) > 1:
        sender_uqID = int(extraData[1])
        if sender_player := frame.get_players().getPlayerByUniqueID(sender_uqID):
            sender_name = sender_player.name
    if len(sender_name) == 0 and (sender_xuid := pkt["XUID"]):
        if sender_player := frame.get_players().getPlayerByXUID(sender_xuid):
            sender_name = sender_player.name

    match pkt["TextType"]:
        case TextType.TextTypeTranslation:
            return None, None, False
        case TextType.TextTypeChat | TextType.TextTypeWhisper:
            src_name = pkt["SourceName"]
            playername = sender_name or src_name
            if src_name == "":
                # /me 消息
                msg_list = msg.split(" ")
                if len(msg_list) >= 3:
                    playername = to_plain_name(playername or msg_list[1])
                    msg = " ".join(msg_list[2:])
                else:
                    fmts.print_war(
                        f"[internal] 无法获取发言中的玩家名与消息: {playername}: {msg}"
                    )
                    return None, None, False
            return playername, msg, sender_name != ""
        case TextType.TextTypeAnnouncement:
            # /say 消息
            src_name = pkt["SourceName"]
            playername = sender_name or pkt["SourceName"]
            msg = msg.removeprefix(f"[{src_name}] ")
            return playername, msg, sender_name != ""
        case TextType.TextTypeObjectWhisper:
            # /tellraw 消息
            return None, None, False
        case _:
            return None, None, False



import os
import platform
import shlex
import subprocess
import threading
import time

from ... import utils
from ...constants import SysStatus
from ...utils import fmts, sys_args, urlmethod
from .neo_libs import file_download as neo_fd, neo_conn
from .neomega_access_point import FrameNeOmgAccessPoint


class FrameNeOmegaLauncher(FrameNeOmgAccessPoint):
    """混合使用ToolDelta和NeOmega启动器启动"""

    launch_type = "NeOmegaLauncher"

    def __init__(self) -> None:
        """初始化 NeOmega 框架

        Args:
            serverNumber (int): 服务器号
            password (str): 服务器密码
            fbToken (str): 验证服务器 Token
            auth_server (str): 验证服务器地址
        """
        super().__init__()
        self.status = SysStatus.LOADING
        self.launch_event = threading.Event()
        self.neomg_proc = None
        self.serverNumber = None
        self.neomega_account_opt = None
        self.bot_name = ""
        self.omega = neo_conn.ThreadOmega(
            connect_type=neo_conn.ConnectType.Remote,
            address="tcp://localhost:24013",
            accountOption=None,
        )
        self.serverNumber: int | None = None
        self.serverPassword: str | None = None
        self.fbToken: str | None = None
        self.auth_server: str | None = None

    def init(self):
        os.makedirs("NeOmega数据", exist_ok=True)
        if "no-download-neomega" not in sys_args.sys_args_to_dict().keys():
            fmts.print_inf("检测依赖库和NeOmega的最新版本..", end="\r")
            try:
                neo_fd.download_neomg()
            except Exception as err:
                raise SystemExit(f"ToolDelta 因下载库异常而退出: {err}") from err
            fmts.print_inf("检测依赖库和NeOmega的最新版本..完成")
        else:
            fmts.print_war("将不会自动检测依赖库和NeOmega的最新版本")
        neo_conn.load_lib()
        self.status = SysStatus.LAUNCHING

    def start_neomega_proc(self) -> int:
        """启动 NeOmega 进程

        Returns:
            int: 端口号
        """
        fmts.print_inf("正在获取空闲端口用于通信..", end="\n")
        free_port = urlmethod.get_free_port(24013)
        sys_machine = platform.uname().machine
        if sys_machine == "x86_64":
            sys_machine = "amd64"
        elif sys_machine == "aarch64":
            sys_machine = "arm64"
        access_point_file = (
            f"omega_launcher_{platform.uname().system.lower()}_{sys_machine}"
        )
        if "TERMUX_VERSION" in os.environ:
            access_point_file = f"omega_launcher_android_{sys_machine}"
        if platform.system() == "Windows":
            access_point_file += ".exe"
        exe_file_path = os.path.join(
            os.getcwd(), "tooldelta", "bin", access_point_file
        )
        if platform.uname().system.lower() == "linux":
            os.system(f"chmod +x {shlex.quote(exe_file_path)}")
        # 只需要+x 即可
        if (
            isinstance(self.serverNumber, type(None))
            or isinstance(self.serverPassword, type(None))
            or isinstance(self.fbToken, type(None))
            or isinstance(self.auth_server, type(None))
        ):
            raise ValueError("未设置服务器号、密码、Token 或验证服务器地址")
        fmts.print_suc(f"将使用空闲端口 §f{free_port}§a 与接入点进行网络通信")
        fmts.print_suc(f"NeOmega 可执行文件路径: {exe_file_path}")
        self.neomg_proc = subprocess.Popen(
            [
                exe_file_path,
                "-server",
                str(self.serverNumber),
                "-T",
                self.fbToken,
                "-access-point-addr",
                f"tcp://localhost:{free_port}",
                "-server-password",
                str(self.serverPassword),
                "-auth-server",
                self.auth_server,
                "-storage-root",
                os.path.join(os.getcwd(), "NeOmega数据"),
            ],
            encoding="utf-8",
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE,
            text=True,
        )
        return free_port

    def _msg_handle_thread(self) -> None:
        """处理来自 NeOmega启动器 的信息"""
        if self.neomg_proc is None or self.neomg_proc.stdout is None:
            raise ValueError("NeOmega进程未启动")
        assert self.neomg_proc.stdin, "标准输入流不可用"
        buffer = ""
        auto_pass = False
        while True:
            char = self.neomg_proc.stdout.read(1)
            if self.neomg_proc.stderr is not None:
                err = self.neomg_proc.stderr.readlines()
                fmts.print_err("\n".join(err))
            if char == "":
                fmts.print_with_info("接入点进程已结束", "§b NOMG ")
                if self.status == SysStatus.LAUNCHING:
                    self.update_status(SysStatus.CRASHED_EXIT)
                break
            if char == "\n":
                msg_orig = buffer
                buffer = ""
                if msg_orig == "SIGNAL: exit":
                    fmts.print_with_info("接入点进程已结束", "§b NOMG ")
                    if self.status == SysStatus.LAUNCHING:
                        self.update_status(SysStatus.CRASHED_EXIT)
                    break
                if "[neOmega 接入点]: 就绪" in msg_orig:
                    self.launch_event.set()
                if msg_orig.startswith(tm_fmt := time.strftime("%H:%M ")):
                    msg_orig = msg_orig.removeprefix(tm_fmt)
                if any(
                    info_type in msg_orig
                    for info_type in [
                        "INFO",
                        "WARNING",
                        "ERROR",
                        "SUCCESS",
                        "\x1b[30;46m\x1b[30;46m      \x1b[0m\x1b[0m",
                    ]
                ):
                    msg_orig = (
                        msg_orig.replace("SUCCESS", "成功")
                        .replace("  ERROR  ", " 错误 ")
                        .replace(" WARNING ", " 警告 ")
                        .replace("  INFO  ", " 消息 ")
                        .strip(" ")
                    )
                    fmts.print_with_info("\b" + msg_orig, "")
                else:
                    fmts.print_with_info(msg_orig, "§b OMGL ")
            else:
                buffer += char
                # 自动通过相同配置文件启动
                if "要使用和上次完全相同的配置启动吗?" in buffer:
                    if not auto_pass:
                        auto_pass = True
                        self.input_to_neomega("")
                        fmts.print_inf(f"{buffer}, 已自动选择为 y")
                # 其他处理, 先独占输入通道, 等待用户输入
                elif (
                    "请输入 y" in buffer and "请输入 n:" in buffer and char != "\n"
                ) or ("请输入" in buffer and ":" in buffer and char != "\n"):
                    msg_orig = buffer.strip()
                    self.input_to_neomega(
                        input(fmts.fmt_info("\b" + msg_orig, "§6 输入 §r"))
                    )
                    buffer = ""

    def input_to_neomega(self, inputter: str) -> None:
        """写入文本到neomega进程"""
        if self.neomg_proc is None or self.neomg_proc.stdout is None:
            raise ValueError("NeOmega进程未启动")
        assert self.neomg_proc.stdin, "标准输入流不可用"
        self.neomg_proc.stdin.write(inputter + "\n")
        self.neomg_proc.stdin.flush()

    def launch(self) -> SystemExit | Exception | SystemError:
        """启动 NeOmega 进程

        returns:
            SystemExit: 正常退出
            Exception: 异常退出
            SystemError: 未知的退出状态
        """
        self.status = SysStatus.LAUNCHING
        openat_port = self.start_neomega_proc()
        fmts.print_load(
            f"NeOmega 数据存放位置: {os.path.join(os.getcwd(), 'tooldelta', 'NeOmega数据')}"
        )
        utils.createThread(
            self._msg_handle_thread,
            usage="处理来自 NeOmega启动器 的信息",
            thread_level=utils.ToolDeltaThread.SYSTEM,
        )
        while not self.launch_event.wait(timeout = 1):
            if self.exit_event.is_set():
                self.update_status(SysStatus.CRASHED_EXIT)
                return SystemError("NeOmage 启动出现问题")
        self.set_omega_conn(f"tcp://127.0.0.1:{openat_port}")
        self.update_status(SysStatus.RUNNING)
        self.start_wait_omega_disconn_thread()
        fmts.print_suc("已获取游戏网络接入点最高权限")
        pcks = [
            self.omega.get_packet_id_to_name_mapping(i)
            for i in self.need_listen_packets
        ]
        self.omega.listen_packets(pcks, self.packet_handler_parent)
        self._exec_launched_listen_cbs()
        fmts.print_suc("接入点注入已就绪")
        self.wait_crashed()
        if self.status == SysStatus.NORMAL_EXIT:
            return SystemExit("正常退出")
        if self.status == SysStatus.CRASHED_EXIT:
            return Exception("接入点进程已崩溃")
        return SystemError("未知的退出状态")



import threading
from typing import Any, TYPE_CHECKING
from collections.abc import Callable

from . import fmts
from .tooldelta_thread import ToolDeltaThread, thread_func

if TYPE_CHECKING:
    from typing import ParamSpec

    PT = ParamSpec("PT")

timer_event_stop = threading.Event()
timer_events_table: dict[int, list[tuple[str, Callable, tuple, dict, int]]] = {}
timer_event_lock = threading.Lock()


def timer_event(
    t: int, name: str | None = None, thread_priority=ToolDeltaThread.PLUGIN
):
    """
    将修饰器下的方法作为一个定时任务, 每隔一段时间被执行一次。
    注意: 请不要在函数内放可能造成堵塞的内容
    注意: 当方法被 timer_event 修饰后, 需要调用一次该方法才能开始定时任务线程!

    Args:
        seconds (int): 周期秒数
        name (Optional[str], optional): 名字, 默认为自动生成的

    ```python
        @timer_event(60, "定时问好")
        def greeting():
            print("Hello!")
        greeting()
    ```
    """

    def receiver(func: "Callable[PT, Any]") -> "Callable[PT, None]":
        def caller(*args, **kwargs):
            func_name = name or f"简易方法:{func.__name__}"
            timer_events_table.setdefault(t, [])
            timer_events_table[t].append(
                (func_name, func, args, kwargs, thread_priority)
            )

        return caller

    return receiver


def timer_events_clear():
    "清理所有定时任务"
    timer_event_lock.acquire()
    for k, funcs_args in timer_events_table.copy().items():
        for func_args in funcs_args:
            (_, _, _, _, priority) = func_args
            if priority != ToolDeltaThread.SYSTEM:
                timer_events_table[k].remove(func_args)
        if timer_events_table[k] == []:
            del timer_events_table[k]
    timer_event_lock.release()


@thread_func("ToolDelta 定时任务", ToolDeltaThread.SYSTEM)
def timer_event_boostrap():
    "启动定时任务, 请不要在系统调用以外调用"
    timer = 0
    timer_event_stop.clear()
    fmts.print_suc("已开始执行 ToolDelta定时任务 函数集.")
    while not timer_event_stop.is_set():
        timer_event_lock.acquire()
        for k, func_args in timer_events_table.copy().items():
            if timer % k == 0:
                for _, caller, args, kwargs, _ in func_args:
                    caller(*args, **kwargs)
        timer_event_lock.release()
        timer_event_stop.wait(1)
        timer += 1



"解析启动参数"

import sys

_cached_sys_args_dict: dict | None = None


def sys_args_to_dict(argv: list[str] = sys.argv) -> dict[str, str | None]:
    """将启动参数转换为字典

    Args:
        argv (list[str], optional): 启动参数

    Returns:
        dict[str, str | None]: 启动参数字典
    """
    # 为什么不用 argsparser? 因为这样可以支持混合型启动参数
    global _cached_sys_args_dict
    if _cached_sys_args_dict is None:
        d: dict[str, str | None] = {}
        i = 0
        while i < len(argv):
            arg = argv[i]
            if arg.startswith("-"):
                arg = arg.strip("--").strip("-")
                try:
                    val = argv[i + 1]
                    if val.startswith("-"):
                        val = None
                    else:
                        i += 1
                except IndexError:
                    val = None
                d[arg] = val
            i += 1
        _cached_sys_args_dict = d
        return d
    else:
        return _cached_sys_args_dict


def print_help():
    print("启动参数帮助：")
    print("    -h, -help  查看帮助 (当前页)")
    print("    -v, --version  获取当前程序版本")
    print("    -title <文本> 在开始界面显示指定的文本 (支持mc彩色符号)")
    print(
        '    -optadd "<选项名>: <执行的cmd>" 添加额外的启动选项, 使用分号分割多个选项'
    )
    print("    -no-update-check  禁用自动更新")
    print("    -no-download-libs  禁用自动更新依赖库")
    print("    -no-download-neomega 在NeOmega混合启动模式下, 禁用自动更新NeOmega启动器")
    print("    -l <启动模式>  按照特定的启动模式，跳过启动页面")
    print("    -user-token <token>  使用传入的而非本地的token来登录验证服务器")


def parse_addopt(opt_str: str):
    arg2cmds: dict[str, str] = {}
    for val in opt_str.split(";"):
        split_res = val.split(":")
        if len(split_res) < 2:
            print('无效的添加选项命令, 应为 -optadd "<选项名>: <执行的cmd>", 如:')
            print('  -optadd "§b管理您的账号: ./acc_mana"')
            print('  -optadd "§b管理您的账号: ./acc_mana; §6自动更新: ./update.sh"')
            raise SystemExit
        opt_str = split_res[0].removeprefix(" ")
        opt_cmd = ":".join(split_res[1:])
        arg2cmds[opt_str] = opt_cmd
    return arg2cmds



import os
import getpass
import requests
from typing import TYPE_CHECKING
from ..auths import fblike_sign_login
from ..constants import tooldelta_cfg, tooldelta_cli
from ..utils import cfg, urlmethod, sys_args, fbtokenFix, if_token, fmts
from .launch_cli import (
    FrameNeOmegaLauncher,
    FrameNeOmgAccessPoint,
    FrameNeOmgAccessPointRemote,
    FrameEulogistLauncher,
    FrameFateArk,
    LAUNCHERS,
    ACCESS_POINT_LAUNCHERS,
)

if TYPE_CHECKING:
    from tooldelta import ToolDelta


"所有启动器框架类型"
LAUNCHERS_SHOWN: list[tuple[str, type[LAUNCHERS]]] = [
    ("NeOmega 框架 (NeOmega 模式，租赁服适应性强，推荐)", FrameNeOmgAccessPoint),
    (
        "NeOmega 框架 (NeOmega 连接模式，需要先启动对应的 neOmega 接入点)",
        FrameNeOmgAccessPointRemote,
    ),
    (
        "NeOmega 框架 (NeOmega 并行模式，同时运行NeOmega和ToolDelta)",
        FrameNeOmegaLauncher,
    ),
    ("Eulogist 框架 (赞颂者和ToolDelta并行使用)", FrameEulogistLauncher),
    ("FateArk 框架", FrameFateArk),
]


class ConfigLoader:
    def __init__(self, frame: "ToolDelta"):
        self.frame = frame

    def load_tooldelta_cfg_and_get_launcher(self) -> LAUNCHERS:
        """加载配置文件"""
        cfg.write_default_cfg_file("ToolDelta基本配置.json", tooldelta_cfg.LAUNCH_CFG)
        try:
            # 读取配置文件
            cfgs = cfg.get_cfg("ToolDelta基本配置.json", tooldelta_cfg.LAUNCH_CFG_STD)
            self.launchMode = cfgs["启动器启动模式(请不要手动更改此项, 改为0可重置)"]
            self.plugin_market_url = cfgs["插件市场源"]
            fmts.publicLogger.switch_logger(cfgs["是否记录日志"])
            if self.launchMode != 0 and self.launchMode not in range(
                1, len(LAUNCHERS_SHOWN) + 1
            ):
                raise cfg.ConfigError("你不该随意修改启动器模式，现在赶紧把它改回 0 吧")
        except cfg.ConfigError as err:
            # 配置文件有误
            r = self.upgrade_cfg()
            if r:
                fmts.print_war("配置文件未升级，已自动升级，请重启 ToolDelta")
            else:
                fmts.print_err(f"ToolDelta 基本配置有误，需要更正：{err}")
            raise SystemExit from err
        # 配置全局 GitHub 镜像 URL
        if cfgs["全局GitHub镜像"] == "":
            cfgs["全局GitHub镜像"] = urlmethod.get_fastest_github_mirror()
            if cfgs["插件市场源"] == "":
                cfgs["插件市场源"] = (
                    cfgs["全局GitHub镜像"]
                    + "/https://raw.githubusercontent.com/ToolDelta-Basic/PluginMarket/main"
                )
            cfg.write_default_cfg_file("ToolDelta基本配置.json", cfgs, True)
        urlmethod.set_global_github_src_url(cfgs["全局GitHub镜像"])

        # 每个启动器框架的单独启动配置之前
        if self.launchMode == 0:
            fmts.print_inf("请选择启动器启动模式 (之后可在 ToolDelta 启动配置更改):")
            for i, (launcher_name, _) in enumerate(LAUNCHERS_SHOWN):
                fmts.print_inf(f" {i + 1} - {launcher_name}")
            while 1:
                try:
                    ch = int(input(fmts.fmt_info("请选择：", "§f 输入 ")))
                    if ch not in range(1, len(LAUNCHERS_SHOWN) + 1):
                        raise ValueError
                    cfgs["启动器启动模式(请不要手动更改此项, 改为0可重置)"] = ch
                    break
                except ValueError:
                    fmts.print_err("输入不合法，或者是不在范围内，请重新输入")
            cfg.write_default_cfg_file("ToolDelta基本配置.json", cfgs, True)
        launcher = LAUNCHERS_SHOWN[
            cfgs["启动器启动模式(请不要手动更改此项, 改为0可重置)"] - 1
        ][1]()
        # 每个启动器框架的单独启动配置
        launcher_config_key = ""
        # 这是 普通 NeOmega 接入点
        if type(launcher) is FrameNeOmgAccessPoint:
            launch_data = cfgs.get(
                "NeOmega接入点启动模式", tooldelta_cfg.LAUNCHER_NEOMEGA_DEFAULT
            )
            launcher_config_key = "NeOmega接入点启动模式"
            try:
                cfg.check_auto(tooldelta_cfg.LAUNCHER_NEOMEGA_STD, launch_data)
            except cfg.ConfigError as err:
                r = self.upgrade_cfg()
                if r:
                    fmts.print_war("配置文件未升级，已自动升级，请重启 ToolDelta")
                else:
                    fmts.print_err(
                        f"ToolDelta 基本配置-NeOmega 启动配置有误，需要更正：{err}"
                    )
                raise SystemExit from err
        # 这是 NeOmega 和 ToolDelta 并行启动
        elif type(launcher) is FrameNeOmegaLauncher:
            launcher_config_key = "NeOmega并行ToolDelta启动模式"
            launch_data = cfgs.get(
                launcher_config_key,
                tooldelta_cfg.LAUNCHER_NEOMG2TD_DEFAULT,
            )
            try:
                cfg.check_auto(tooldelta_cfg.LAUNCHER_NEOMG2TD_STD, launch_data)
            except cfg.ConfigError as err:
                r = self.upgrade_cfg()
                if r:
                    fmts.print_war("配置文件未升级，已自动升级，请重启 ToolDelta")
                else:
                    fmts.print_err(
                        f"ToolDelta 基本配置-NeOmega 启动配置有误，需要更正：{err}"
                    )
                raise SystemExit from err
        elif type(launcher) is FrameFateArk:
            launcher_config_key = "FateArk接入点启动模式"
            launch_data = cfgs.get(
                launcher_config_key, tooldelta_cfg.LAUNCHER_FATEARK_DEFAULT
            )
            try:
                cfg.check_auto(tooldelta_cfg.LAUNCHER_FATEARK_STD, launch_data)
            except cfg.ConfigError as err:
                r = self.upgrade_cfg()
                if r:
                    fmts.print_war("配置文件未升级，已自动升级，请重启 ToolDelta")
                else:
                    fmts.print_err(
                        f"ToolDelta 基本配置-FateArk 启动配置有误，需要更正：{err}"
                    )
                raise SystemExit from err
        elif type(launcher) is FrameNeOmgAccessPointRemote:
            launcher_config_key = "NeOmega远程接入点模式"
            launch_data = cfgs.get(
                launcher_config_key, tooldelta_cfg.LAUNCHER_NEOMEGARM_DEFAULT
            )
            cfgs[launcher_config_key] = launch_data
            cfg.write_default_cfg_file("ToolDelta基本配置.json", cfgs, True)
            try:
                cfg.check_auto(tooldelta_cfg.LAUNCHER_NEOMEGARM_STD, launch_data)
            except cfg.ConfigError as err:
                r = self.upgrade_cfg()
                if r:
                    fmts.print_war("配置文件未升级，已自动升级，请重启 ToolDelta")
                else:
                    fmts.print_err(
                        f"ToolDelta 基本配置-远程NeOmega 启动配置有误，需要更正：{err}"
                    )
        elif type(launcher) is FrameEulogistLauncher:
            # 不需要任何配置文件
            ...
        else:
            raise ValueError("LAUNCHER Error")

        if type(launcher) in ACCESS_POINT_LAUNCHERS:
            serverNumber = launch_data["服务器号"]
            serverPasswd: str = launch_data["密码"]
            auth_server = launch_data.get("验证服务器地址(更换时记得更改fbtoken)", "")
            if serverNumber == 0:
                while 1:
                    try:
                        serverNumber = int(
                            input(fmts.fmt_info("请输入租赁服号：", "§b 输入 "))
                        )
                        serverPasswd = (
                            getpass.getpass(
                                fmts.fmt_info(
                                    "请输入租赁服密码 (不会回显，没有请直接回车): ",
                                    "§b 输入 ",
                                )
                            )
                            or ""
                        )
                        launch_data["服务器号"] = int(serverNumber)
                        launch_data["密码"] = serverPasswd
                        cfgs[launcher_config_key] = launch_data
                        cfg.write_default_cfg_file("ToolDelta基本配置.json", cfgs, True)
                        fmts.print_suc("登录配置设置成功")
                        break
                    except ValueError:
                        fmts.print_err("输入有误，租赁服号和密码应当是纯数字")
            auth_servers = tooldelta_cli.AUTH_SERVERS
            if auth_server == "":
                fmts.print_inf("选择 ToolDelta 机器人账号 使用的验证服务器：")
                i = 0
                for i, (auth_server_name, _) in enumerate(auth_servers):
                    fmts.print_inf(f" {i + 1} - {auth_server_name}")
                fmts.print_inf(f" {i + 2} - 手动输入")
                fmts.print_inf(
                    "§cNOTE: 使用的机器人账号是在哪里获取的就选择哪一个验证服务器，不能混用"
                )
                while 1:
                    try:
                        ch = int(input(fmts.fmt_info("请选择: ", "§f 输入 ")))
                        if ch not in range(1, len(auth_servers) + 1):
                            if ch == len(auth_servers) + 1:
                                auth_server = input(
                                    fmts.fmt_info(
                                        "请手动输入验证服务器地址: ", "§f 输入 "
                                    )
                                )
                                cfgs[launcher_config_key][
                                    "验证服务器地址(更换时记得更改fbtoken)"
                                ] = auth_server
                                break
                        else:
                            auth_server = auth_servers[ch - 1][1]
                            cfgs[launcher_config_key][
                                "验证服务器地址(更换时记得更改fbtoken)"
                            ] = auth_server
                            break
                    except ValueError:
                        fmts.print_err("输入不合法，或者是不在范围内，请重新输入")
                cfg.write_default_cfg_file("ToolDelta基本配置.json", cfgs, True)
            # 读取 token
            if not (fbtoken := sys_args.sys_args_to_dict().get("user-token")):
                if not os.path.isfile("fbtoken"):
                    fmts.print_inf(
                        "请选择登录方法:\n 1 - 使用账号密码获取 fbtoken\n 2 - 手动输入 fbtoken\r"
                    )
                    login_method = input(fmts.fmt_info("请输入你的选择：", "§6 输入 "))
                    while True:
                        if login_method.isdigit() is False or int(
                            login_method
                        ) not in range(1, 3):
                            login_method = input(
                                fmts.fmt_info(
                                    "输入有误, 请输入正确的序号：", "§6 警告 "
                                )
                            )
                        else:
                            break
                    if login_method == "1":
                        try:
                            token = fblike_sign_login(
                                cfgs[launcher_config_key][
                                    "验证服务器地址(更换时记得更改fbtoken)"
                                ],
                                tooldelta_cli.FBLIKE_APIS,
                            )
                            with open("fbtoken", "w", encoding="utf-8") as f:
                                f.write(token)
                        except requests.exceptions.RequestException as e:
                            fmts.print_err(
                                f"登录失败, 原因: {e}\n请尝试选择手动输入 fbtoken"
                            )
                            raise SystemExit
                if_token()
                fbtokenFix()
                with open("fbtoken", encoding="utf-8") as f:
                    fbtoken = f.read()
            if isinstance(launcher, ACCESS_POINT_LAUNCHERS):
                launcher.set_launch_data(
                    serverNumber, serverPasswd, fbtoken, auth_server
                )
        fmts.print_suc("配置文件读取完成")
        return launcher

    @staticmethod
    def upgrade_cfg() -> bool:
        """升级配置文件

        Returns:
            bool: 是否升级了配置文件
        """
        old_cfg: dict = cfg.get_cfg("ToolDelta基本配置.json", {})
        old_cfg_keys = old_cfg.keys()
        need_upgrade_cfg = False
        for k, v in tooldelta_cfg.LAUNCH_CFG.items():
            if k not in old_cfg_keys:
                old_cfg[k] = v
                need_upgrade_cfg = True
        if need_upgrade_cfg:
            cfg.write_default_cfg_file("ToolDelta基本配置.json", old_cfg, True)
        return need_upgrade_cfg

    @staticmethod
    def change_config():
        """修改配置文件"""
        try:
            old_cfg = cfg.get_cfg(
                "ToolDelta基本配置.json", tooldelta_cfg.LAUNCH_CFG_STD
            )
        except FileNotFoundError:
            fmts.clean_print("§c未初始化配置文件, 无法进行修改")
            return
        except cfg.ConfigError as err:
            fmts.print_err(f"配置文件损坏：{err}")
            return
        if (
            old_cfg["启动器启动模式(请不要手动更改此项, 改为0可重置)"] - 1
        ) not in range(0, 2):
            fmts.print_err(
                f"配置文件损坏：启动模式错误：{old_cfg['启动器启动模式(请不要手动更改此项, 改为0可重置)']}"
            )
            return
        while 1:
            md = (
                "NeOmega 框架 (NeOmega 模式，租赁服适应性强，推荐)",
                "NeOmega 框架 (NeOmega 连接模式，需要先启动对应的 neOmega 接入点)",
                "混合启动模式 (同一个机器人同时启动 ToolDelta 和 NeOmega)",
                "Eulogist 框架 (赞颂者和 ToolDelta 并行运行)",
            )
            fmts.clean_print("§b现有配置项如下:")
            fmts.clean_print(
                f" 1. 启动器启动模式：{md[old_cfg['启动器启动模式(请不要手动更改此项, 改为0可重置)'] - 1]}"
            )
            fmts.clean_print(f" 2. 是否记录日志：{old_cfg['是否记录日志']}")
            fmts.clean_print("    §a直接回车: 保存并退出")
            resp = input(fmts.clean_fmt("§6输入序号可修改配置项(0~4): ")).strip()
            if resp == "":
                cfg.write_default_cfg_file("ToolDelta基本配置.json", old_cfg, True)
                fmts.clean_print("§a配置已保存!")
                return
            match resp:
                case "1":
                    fmts.print_inf(
                        "选择启动器启动模式 (之后可在 ToolDelta 启动配置更改):"
                    )
                    for i, (launcher_name, _) in enumerate(LAUNCHERS_SHOWN):
                        fmts.print_inf(f" {i + 1} - {launcher_name}")
                    while 1:
                        try:
                            ch = int(input(fmts.clean_fmt("请选择：")))
                            if ch not in range(1, len(LAUNCHERS_SHOWN) + 1):
                                raise ValueError
                            old_cfg[
                                "启动器启动模式(请不要手动更改此项, 改为0可重置)"
                            ] = ch
                            break
                        except ValueError:
                            fmts.print_err("输入不合法，或者是不在范围内，请重新输入")
                            continue
                    input(
                        fmts.clean_fmt(
                            f"§a已选择启动器启动模式：§f{md[old_cfg['启动器启动模式(请不要手动更改此项, 改为0可重置)'] - 1]}, 回车键继续"
                        )
                    )
                case "2":
                    old_cfg["是否记录日志"] = [True, False][old_cfg["是否记录日志"]]
                    input(
                        fmts.clean_fmt(
                            f"日志记录模式已改为：{['§c关闭', '§a开启'][old_cfg['是否记录日志']]}, 回车键继续"
                        )
                    )



import json
import traceback
from dataclasses import dataclass
from typing import Any, TYPE_CHECKING
from collections.abc import Callable
from .. import plugin_market
from ..constants import SysStatus
from ..utils import fmts, thread_func, ToolDeltaThread
from .launch_cli import FrameNeOmegaLauncher, FrameNeOmgAccessPoint


if TYPE_CHECKING:
    from tooldelta import ToolDelta


@dataclass
class CommandTrigger:
    triggers: list[str]
    argument_hint: str | None
    usage: str
    cb: Callable[[list[str]], Any]

    def __hash__(self):
        return id(self)


class ConsoleCmdManager:
    def __init__(self, frame: "ToolDelta") -> None:
        self.frame = frame
        self.commands: dict[str, CommandTrigger] = {}

    def add_console_cmd_trigger(
        self,
        triggers: list[str],
        arg_hint: str | None,
        usage: str,
        func: Callable[[list[str]], Any],
    ):
        """注册 ToolDelta 控制台的菜单项

        Args:
            triggers (list[str]): 触发词列表
            arg_hint (str | None): 菜单命令参数提示句
            usage (str): 命令说明
            func (Callable[[list[str]], None]): 菜单回调方法
        """
        trig = CommandTrigger(triggers, arg_hint, usage, func)
        for trigger in triggers:
            self.commands[self.test_duplicate_trigger(trigger)] = trig

    def get_cmd_triggers(self):
        return list(self.commands.values())

    def execute_cmd(self, cmd: str) -> bool:
        cmd = cmd.strip()
        cmd_finded = False
        for prefix, trig in self.commands.copy().items():
            if cmd.startswith(prefix):
                cmds = cmd.removeprefix(prefix).split()
                res = trig.cb(cmds)
                cmd_finded = True
                if res is True:
                    return True
        if not cmd_finded and cmd:
            fmts.print_war(f"命令 {cmd.split()[0]} 不存在, 输入 ? 查看帮助")
        return False

    def test_duplicate_trigger(self, trigger: str):
        for exists_trigger in self.commands.keys():
            counter = 0
            invalid = False
            origin_trigger = trigger
            while 1:
                if not (trigger.startswith(exists_trigger) or exists_trigger.startswith(trigger)):
                    if invalid:
                        fmts.print_war(
                            f"命令 {origin_trigger} 与 {exists_trigger} 冲突, 已更改为 {trigger}"
                        )
                        return trigger
                    break
                invalid = True
                counter += 1
                trigger = f"{counter}-{trigger}"
        return trigger

    @thread_func("控制台执行命令", ToolDeltaThread.SYSTEM)
    def command_readline_proc(self):
        fmts.print_suc("ToolHack Terminal 进程已注入, 允许开启标准输入")
        while 1:
            try:
                try:
                    rsp = input()
                    if rsp in ("^C", "^D"):
                        raise KeyboardInterrupt
                except (KeyboardInterrupt, EOFError):
                    fmts.print_inf("按退出键退出中...")
                    self.frame.launcher.update_status(SysStatus.NORMAL_EXIT)
                    return
                self.execute_cmd(rsp)
            except (EOFError, KeyboardInterrupt):
                fmts.print_war("命令执行被中止")
            except Exception:
                fmts.print_err(f"控制台指令执行出现问题: {traceback.format_exc()}")
                fmts.print_err("§6虽然出现了问题, 但是您仍然可以继续使用控制台菜单")

    def prepare_internal_cmds(self):
        @thread_func("控制台执行指令并获取回调", ToolDeltaThread.SYSTEM)
        def _execute_mc_command_and_get_callback(cmds: list[str]) -> None:
            """执行 Minecraft 指令并获取回调结果。

            Args:
                cmd (str): 要执行的 Minecraft 指令。

            Raises:
                ValueError: 当指令执行失败时抛出。
            """
            cmd = " ".join(cmds)
            try:
                result = self.frame.get_game_control().sendcmd_with_resp(cmd, 10)
                if (result.OutputMessages[0].Message == "commands.generic.syntax") | (
                    result.OutputMessages[0].Message == "commands.generic.unknown"
                ):
                    fmts.print_err(f'未知的 MC 指令, 可能是指令格式有误: "{cmd}"')
                else:
                    if (
                        game_text_handler
                        := self.frame.get_game_control().game_data_handler
                    ):
                        msgs_output = " ".join(
                            json.loads(i)
                            for i in game_text_handler.Handle_Text_Class1(
                                result.as_dict["OutputMessages"]
                            )
                        )
                    else:
                        msgs_output = json.dumps(
                            result.as_dict["OutputMessages"],
                            indent=2,
                            ensure_ascii=False,
                        )
                    desc = json.dumps(
                        result.OutputMessages[0].Parameters,
                        indent=2,
                        ensure_ascii=False,
                    )
                    if result.SuccessCount:
                        fmts.print_suc("指令执行成功: " + msgs_output)
                        fmts.print_suc(desc)
                    else:
                        fmts.print_war("指令执行失败: " + msgs_output)
                        fmts.print_war(desc)
            except IndexError as exec_err:
                if isinstance(result, type(None)):
                    raise ValueError("指令执行失败") from exec_err
                if result.SuccessCount:
                    fmts.print_suc(
                        f"指令执行成功, 详细返回结果:\n{json.dumps(result.as_dict['OutputMessages'], indent=2, ensure_ascii=False)}"
                    )
            except TimeoutError:
                fmts.print_err(f"[超时] 指令获取结果返回超时: {cmd}")

        def _send_to_neomega(cmds: list[str]):
            # 仅当启动模式为 neomega 并行模式才生效
            assert isinstance(self.frame.launcher, FrameNeOmgAccessPoint)
            assert self.frame.launcher.neomg_proc
            assert self.frame.launcher.neomg_proc.stdin
            self.frame.launcher.neomg_proc.stdin.write(" ".join(cmds) + "\n")
            self.frame.launcher.neomg_proc.stdin.flush()

        def say(msgs: list[str]):
            self.frame.get_game_control().say_to(
                "@a", f"[§b控制台§r] §3{' '.join(msgs)}"
            )

        def _basic_help(_):
            menu = self.get_cmd_triggers()
            fmts.print_inf("§a以下是可选的菜单指令项: ")
            for cmd_trigger in set(menu):
                if cmd_trigger.argument_hint:
                    fmts.print_inf(
                        f" §e{' 或 '.join(cmd_trigger.triggers)} §b{cmd_trigger.argument_hint} §f->  {cmd_trigger.usage}"
                    )
                else:
                    fmts.print_inf(
                        f" §e{' 或 '.join(cmd_trigger.triggers)}  §f->  {cmd_trigger.usage}"
                    )

        def _exit(_):
            fmts.print_inf("准备退出..")
            self.frame.launcher.update_status(SysStatus.NORMAL_EXIT)

        self.add_console_cmd_trigger(
            ["?", "help", "帮助", "？"],
            None,
            "查询可用菜单指令",
            _basic_help,
        )
        self.add_console_cmd_trigger(
            ["exit"],
            None,
            "退出并关闭ToolDelta",
            _exit,
        )
        self.add_console_cmd_trigger(
            ["插件市场"],
            None,
            "进入插件市场",
            lambda _: plugin_market.market.enter_plugin_market(in_game=True),
        )
        self.add_console_cmd_trigger(
            ["/"], "[指令]", "执行 MC 指令", lambda args: _execute_mc_command_and_get_callback(args) and None
        )
        self.add_console_cmd_trigger(
            ["list"],
            None,
            "查询在线玩家",
            lambda _: fmts.print_inf(
                "在线玩家：" + ", ".join(self.frame.get_game_control().allplayers)
            ),
        )
        self.add_console_cmd_trigger(
            ["reload"],
            None,
            "重载插件 (可能有部分特殊插件无法重载)",
            lambda _: self.frame.reload(),
        )
        if isinstance(self.frame.launcher, FrameNeOmegaLauncher):
            self.add_console_cmd_trigger(
                ["o"], "neomega命令", "执行neomega控制台命令", _send_to_neomega
            )
        self.add_console_cmd_trigger(["!"], "[消息]", "在游戏内广播消息", say)

    def reset_cmds(self):
        self.commands.clear()
        self.prepare_internal_cmds()

    def start_proc_thread(self):
        self.prepare_internal_cmds()
        self.command_readline_proc()



import numpy
from tooldelta.internal.launch_cli.neo_libs.blob_hash.define import (
    BLOCKING_DEADLINE_SECONDS,
    BaseBlobHashHolder,
)
from tooldelta.internal.launch_cli.neo_libs.blob_hash.mirror_world_handler import (
    MirrorWorldHandler,
)
from tooldelta.internal.launch_cli.neo_libs.blob_hash.packet.define import (
    HashWithPosition,
    SubChunkPos,
)
from tooldelta.internal.launch_cli.neo_libs.blob_hash.packet.server_and_client import (
    SetHolderRequest,
    SetHolderResponse,
)
from tooldelta.internal.launch_cli.neo_libs.blob_hash.packet.server_and_mirror_world import (
    GetDiskHashPayload,
    GetDiskHashPayloadResponse,
    QueryDiskHashExist,
    QueryDiskHashExistResponse,
    RequireSyncHashToDisk,
    ServerDisconnected,
)
from tooldelta.mc_bytes_packet.sub_chunk import (
    SUB_CHUNK_RESULT_SUCCESS_ALL_AIR,
    SubChunk,
)
from tooldelta.utils.tooldelta_thread import ToolDeltaThread


class MirrorWorldListener:
    """
    MirrorWorldListener 是基于 MirrorWorldHandler 实现的监听器，
    然后镜像存档的持有人便可作为资源中心处理来自服务者的资源请求
    """

    mirror_world_handler: MirrorWorldHandler
    _has_register: bool

    def __init__(self, mirror_world_handler: MirrorWorldHandler):
        """
        Args:
            mirror_world_handler (MirrorWorldHandler):
                镜像存档资源持有人所使用的处理函数，
                它们被用于处理来自客户端的 blob cache 查询和同步请求
        """
        self.mirror_world_handler = mirror_world_handler
        self._has_register = False

    def register_listener(self) -> bool:
        """
        register_listener 为镜像资源持有者注册其所使用的监听器，
        register_listener 应当最多被调用一次。

        如果已经注册了监听器，或当前结点不是镜像存档的持有人，
        则返回假；否则，返回真
        """
        if self._has_register or not self._base_blob_hash_holder().is_disk_holder:
            return False
        self._register_listener()
        self._has_register = True
        return True

    def _base_blob_hash_holder(self) -> BaseBlobHashHolder:
        return self.mirror_world_handler.base_blob_hash_holder

    def _register_listener(self):
        # MC Packet
        self._base_blob_hash_holder().omega.listen_packets(
            "SubChunk", self._on_sub_chunk, False
        )
        # Handle server keep alive and disconnected
        self._base_blob_hash_holder().omega.soft_reg(
            "blob-hash-keep-alive", True, self._handle_keep_alive
        )
        self._base_blob_hash_holder().omega.soft_listen(
            "blob-hash-server-disconnected", True, self._handle_server_disconnected
        )
        # Handle server request
        self._base_blob_hash_holder().omega.soft_reg(
            "blob-hash-query-disk-hash-exist", True, self._handle_query_disk_hash_exist
        )
        self._base_blob_hash_holder().omega.soft_reg(
            "blob-hash-get-disk-hash-payload", True, self._handle_get_disk_hash_payload
        )
        self._base_blob_hash_holder().omega.soft_reg(
            "blob-hash-require-sync-hash-to-disk",
            True,
            self._handle_require_sync_hash_to_disk,
        )

    def _on_sub_chunk(self, _: str, bs: bytes):
        if self.mirror_world_handler.f4 is None:
            return

        pos: list[HashWithPosition] = []

        s = SubChunk()
        s.decode(bs)

        for i in s.Entries:
            if i.Result == SUB_CHUNK_RESULT_SUCCESS_ALL_AIR:
                pos.append(
                    HashWithPosition(
                        0,
                        SubChunkPos(i.SubChunkPosX, i.SubChunkPosY, i.SubChunkPosZ),
                        s.Dimension,
                    )
                )

        if len(pos) > 0:
            self.mirror_world_handler.f4(pos)

    def _handle_keep_alive(self, packet: bytes) -> bytes:
        return packet

    def _handle_server_disconnected(self, packet: bytes):
        pk = ServerDisconnected()
        pk.decode(packet)

        if (
            pk.mirror_world_holder_name
            != self._base_blob_hash_holder().disk_holder_name
        ):
            return

        self._base_blob_hash_holder().disk_holder_name = ""
        self._base_blob_hash_holder().is_disk_holder = False

        # Server thought we were dead,
        # however, we are still alive,
        # so we try to re-set as holder
        # again.
        def recover():
            # If re-set request failed,
            # then we lost the status of
            # mirror world holder.
            recover_pk = SetHolderRequest()
            recover_states: bytes | None = (
                self._base_blob_hash_holder().omega.soft_call_with_bytes(
                    recover_pk.packet_name,
                    recover_pk.encode(),
                    timeout=BLOCKING_DEADLINE_SECONDS,
                )
            )

            if recover_states is None and self.mirror_world_handler.f5 is not None:
                self.mirror_world_handler.f5()

            resp = SetHolderResponse()
            resp.decode(recover_states)

            if resp.success_states:
                self._base_blob_hash_holder().disk_holder_name = resp.holder_name
                self._base_blob_hash_holder().is_disk_holder = True
            elif self.mirror_world_handler.f5 is not None:
                self.mirror_world_handler.f5()

        ToolDeltaThread(recover, usage="Mirror World Holder Recover Thread")

    def _handle_query_disk_hash_exist(self, packet: bytes) -> bytes | None:
        if (
            not self._base_blob_hash_holder().is_disk_holder
            or self.mirror_world_handler.f1 is None
        ):
            return

        pk = QueryDiskHashExist()
        pk.decode(packet)

        resp = QueryDiskHashExistResponse(
            self._base_blob_hash_holder().disk_holder_name,
            numpy.array(self.mirror_world_handler.f1(pk.hashes), dtype=numpy.bool),
        )
        return resp.encode()

    def _handle_get_disk_hash_payload(self, packet: bytes) -> bytes | None:
        if (
            not self._base_blob_hash_holder().is_disk_holder
            or self.mirror_world_handler.f2 is None
        ):
            return

        pk = GetDiskHashPayload()
        pk.decode(packet)

        resp = GetDiskHashPayloadResponse(
            self._base_blob_hash_holder().disk_holder_name,
            self.mirror_world_handler.f2(pk.hashes),
        )
        return resp.encode()

    def _handle_require_sync_hash_to_disk(self, packet: bytes) -> bytes | None:
        if (
            not self._base_blob_hash_holder().is_disk_holder
            or self.mirror_world_handler.f3 is None
        ):
            return

        pk = RequireSyncHashToDisk()
        pk.decode(packet)

        self.mirror_world_handler.f3(pk.payload)
        return b""



from typing import TYPE_CHECKING
from collections.abc import Callable

from ..mc_bytes_packet.base_bytes_packet import BaseBytesPacket
from ..constants.packets import PacketIDS
from ..constants.tooldelta_cli import INTERNAL_LISTEN_PACKETS
from ..utils import create_result_cb

if TYPE_CHECKING:
    from tooldelta.frame import ToolDelta


DictPacketListener = Callable[[dict], bool]
BytesPacketListener = Callable[[BaseBytesPacket], bool]
DictPacketWaiter = Callable[[dict], None]


class PacketHandler:
    def __init__(self, frame: "ToolDelta"):
        self.frame = frame
        self.listen_packets = INTERNAL_LISTEN_PACKETS.copy()
        self.dict_packet_listener_with_priority: dict[
            int, dict[int, list[DictPacketListener]]
        ] = {}
        self.bytes_packet_listener_with_priority: dict[
            int, dict[int, list[BytesPacketListener]]
        ] = {}
        self.dict_packet_wait_callbacks: dict[int, list[DictPacketWaiter]] = {}

    def wait_next_packet(self, packet_id: PacketIDS, timeout: float = 30) -> dict | None:
        getter, setter = create_result_cb(dict)
        self.dict_packet_wait_callbacks.setdefault(packet_id, []).append(setter)
        res = getter(timeout)
        self.dict_packet_wait_callbacks[packet_id].remove(setter)
        return res

    def add_dict_packet_listener(
        self,
        packet_id: PacketIDS,
        cb: DictPacketListener,
        priority: int = 0,
    ):
        self.listen_packets.add(packet_id)

        self.dict_packet_listener_with_priority.setdefault(packet_id, {})
        self.dict_packet_listener_with_priority[packet_id].setdefault(priority, [])

        plist = self.dict_packet_listener_with_priority[packet_id][priority]
        if cb not in plist:
            plist.append(cb)

    def add_bytes_packet_listener(
        self,
        packet_id: PacketIDS,
        cb: BytesPacketListener,
        priority: int = 0,
    ):
        self.listen_packets.add(packet_id)

        self.bytes_packet_listener_with_priority.setdefault(packet_id, {})
        self.bytes_packet_listener_with_priority[packet_id].setdefault(priority, [])

        plist = self.bytes_packet_listener_with_priority[packet_id][priority]
        if cb not in plist:
            plist.append(cb)

    def entrance_dict_packet(self, packetID: int, packet: dict):
        self._handle_dict_packet(packetID, packet)

    def entrance_bytes_packet(self, packetID: int, packet: BaseBytesPacket):
        self._handle_bytes_packet(packetID, packet)

    def _handle_dict_packet(self, packetID: int, packet: dict):
        pkt_cbs = self.dict_packet_listener_with_priority.get(packetID)
        if pkt_cbs:
            for priority in sorted(pkt_cbs.keys(), reverse=True):
                for cb in pkt_cbs[priority]:
                    if cb(packet):
                        return

    def _handle_bytes_packet(self, packetID: int, packet: BaseBytesPacket):
        pkt_cbs = self.bytes_packet_listener_with_priority.get(packetID)
        if pkt_cbs:
            for priority in sorted(pkt_cbs.keys(), reverse=True):
                for cb in pkt_cbs[priority]:
                    if cb(packet):
                        return

    def _handle_next_dict_packet(self, packetID: int, packet: dict):
        cbs = self.dict_packet_wait_callbacks.get(packetID, [])
        for cb in cbs.copy():
            cb(packet)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: proto/command.proto
# Protobuf Python Version: 5.29.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    29,
    0,
    '',
    'proto/command.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import response_pb2 as proto_dot_response__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x13proto/command.proto\x12\x15\x66\x61teark.proto.command\x1a\x14proto/response.proto\"#\n\x14SendWOCommandRequest\x12\x0b\n\x03\x63md\x18\x01 \x01(\t\"#\n\x14SendWSCommandRequest\x12\x0b\n\x03\x63md\x18\x01 \x01(\t\"\'\n\x18SendPlayerCommandRequest\x12\x0b\n\x03\x63md\x18\x01 \x01(\t\"7\n\x14SendAICommandRequest\x12\x12\n\nruntime_id\x18\x01 \x01(\t\x12\x0b\n\x03\x63md\x18\x02 \x01(\t\"/\n SendWSCommandWithResponseRequest\x12\x0b\n\x03\x63md\x18\x01 \x01(\t\"3\n$SendPlayerCommandWithResponseRequest\x12\x0b\n\x03\x63md\x18\x01 \x01(\t\"C\n SendAICommandWithResponseRequest\x12\x12\n\nruntime_id\x18\x01 \x01(\t\x12\x0b\n\x03\x63md\x18\x02 \x01(\t2\xba\x06\n\x0e\x43ommandService\x12\x65\n\rSendWOCommand\x12+.fateark.proto.command.SendWOCommandRequest\x1a\'.fateark.proto.response.GeneralResponse\x12\x65\n\rSendWSCommand\x12+.fateark.proto.command.SendWSCommandRequest\x1a\'.fateark.proto.response.GeneralResponse\x12m\n\x11SendPlayerCommand\x12/.fateark.proto.command.SendPlayerCommandRequest\x1a\'.fateark.proto.response.GeneralResponse\x12\x65\n\rSendAICommand\x12+.fateark.proto.command.SendAICommandRequest\x1a\'.fateark.proto.response.GeneralResponse\x12}\n\x19SendWSCommandWithResponse\x12\x37.fateark.proto.command.SendWSCommandWithResponseRequest\x1a\'.fateark.proto.response.GeneralResponse\x12\x85\x01\n\x1dSendPlayerCommandWithResponse\x12;.fateark.proto.command.SendPlayerCommandWithResponseRequest\x1a\'.fateark.proto.response.GeneralResponse\x12}\n\x19SendAICommandWithResponse\x12\x37.fateark.proto.command.SendAICommandWithResponseRequest\x1a\'.fateark.proto.response.GeneralResponseB\x1fZ\x1d./proto/gen/command;commandpbb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'proto.command_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'Z\035./proto/gen/command;commandpb'
  _globals['_SENDWOCOMMANDREQUEST']._serialized_start=68
  _globals['_SENDWOCOMMANDREQUEST']._serialized_end=103
  _globals['_SENDWSCOMMANDREQUEST']._serialized_start=105
  _globals['_SENDWSCOMMANDREQUEST']._serialized_end=140
  _globals['_SENDPLAYERCOMMANDREQUEST']._serialized_start=142
  _globals['_SENDPLAYERCOMMANDREQUEST']._serialized_end=181
  _globals['_SENDAICOMMANDREQUEST']._serialized_start=183
  _globals['_SENDAICOMMANDREQUEST']._serialized_end=238
  _globals['_SENDWSCOMMANDWITHRESPONSEREQUEST']._serialized_start=240
  _globals['_SENDWSCOMMANDWITHRESPONSEREQUEST']._serialized_end=287
  _globals['_SENDPLAYERCOMMANDWITHRESPONSEREQUEST']._serialized_start=289
  _globals['_SENDPLAYERCOMMANDWITHRESPONSEREQUEST']._serialized_end=340
  _globals['_SENDAICOMMANDWITHRESPONSEREQUEST']._serialized_start=342
  _globals['_SENDAICOMMANDWITHRESPONSEREQUEST']._serialized_end=409
  _globals['_COMMANDSERVICE']._serialized_start=412
  _globals['_COMMANDSERVICE']._serialized_end=1238
# @@protoc_insertion_point(module_scope)



"数据包类构建器"

from tooldelta.constants import PacketIDS
from tooldelta.protocol.reader import Reader


class Packet:
    ID: int
    raw: bytes

    def __init__(self, raw: bytes):
        self.raw = raw

    def marshal(self):
        raise NotImplementedError


class SubPacket_CmdOutputMsg:
    """命令输出消息子包构建"""

    Success: bool
    Message: str
    Parameters: list

    def __init__(self, pkt: dict):
        self.Success = pkt["Success"]
        self.Parameters = pkt["Parameters"]
        self.Message = pkt["Message"]


class SubPacket_CmdOrigin:
    "命令来源子包构建"

    Origin: int
    UUID: str
    RequestID: str
    PlayerUniqueID: int

    def __init__(self, pkt: dict):
        self.Origin = pkt["Origin"]
        self.UUID = pkt["UUID"]
        self.RequestID = pkt["RequestID"]
        self.PlayerUniqueID = pkt["PlayerUniqueID"]


class Packet_CommandOutput:
    "命令输出包构建"

    CommandOrigin: SubPacket_CmdOrigin
    OutputType: int
    SuccessCount: int
    OutputMessages: list[SubPacket_CmdOutputMsg]
    as_dict: dict

    def __init__(self, pkt: dict):
        self.as_dict = pkt
        self.CommandOrigin = SubPacket_CmdOrigin(pkt["CommandOrigin"])
        self.OutputMessages = [
            SubPacket_CmdOutputMsg(imsg) for imsg in pkt["OutputMessages"]
        ]
        self.SuccessCount = pkt["SuccessCount"]
        self.OutputType = pkt["OutputType"]


class Text(Packet):
    ID = PacketIDS.IDText
    TextType: int
    NeedsTranslation: bool
    SourceName: str
    Message: str
    Parameters: list[str]
    XUID: str
    PlatformChatID: str
    NeteaseExtraData: list[str]
    Unknown: str

    def marshal(self):
        reader = Reader(self.raw)
        self.TextType = reader.uint8()
        self.NeedsTranslation = reader.bool()
        match self.TextType:
            case 1 | 7 | 8:
                self.SourceName = reader.string()
                self.Message = reader.string()
                self.Parameters = []
            case 0 | 5 | 6 | 9 | 10 | 11:
                self.Message = reader.string()
            case 2 | 3 | 4:
                self.Message = reader.string()
                self.Parameters = reader.list(reader.string)
        self.XUID = reader.string()
        self.PlatformChatID = reader.string()
        # 网易
        match self.TextType:
            case 1:
                self.NeteaseExtraData = reader.list(reader.string)
            case 3:
                self.Unknown = reader.string()




import struct
import numpy
import uuid

from dataclasses import dataclass, field
from io import BytesIO
from tooldelta.internal.launch_cli.neo_libs.blob_hash.packet.define import (
    HashWithPosition,
    PayloadByHash,
    decode_hash_with_position,
)


@dataclass
class KeepAlive:
    uid: uuid.UUID = field(
        default_factory=lambda: uuid.UUID("00000000-0000-0000-0000-000000000000")
    )
    packet_name: str = "blob-hash-keep-alive"

    def decode(self, bs: bytes | None):
        if bs is None:
            return
        reader = BytesIO(bs)
        self.uid = uuid.UUID(
            reader.read(struct.unpack("<h", reader.read(2))[0]).decode(encoding="utf-8")
        )


@dataclass
class ServerDisconnected:
    mirror_world_holder_name: str = ""
    packet_name: str = "blob-hash-server-disconnected"

    def decode(self, bs: bytes | None):
        if bs is None:
            return
        reader = BytesIO(bs)
        self.mirror_world_holder_name = reader.read(
            struct.unpack("<h", reader.read(2))[0]
        ).decode(encoding="utf-8")


@dataclass
class QueryDiskHashExist:
    hashes: list[HashWithPosition] = field(default_factory=lambda: [])
    packet_name: str = "blob-hash-query-disk-hash-exist"

    def decode(self, bs: bytes | None):
        if bs is None:
            return
        reader = BytesIO(bs)
        self.hashes = [
            decode_hash_with_position(reader)
            for _ in range(struct.unpack("<H", reader.read(2))[0])
        ]


@dataclass
class QueryDiskHashExistResponse:
    holder_name: str = ""
    states: numpy.ndarray = field(
        default_factory=lambda: numpy.array([], dtype=numpy.bool)
    )
    packet_name: str = "blob-hash-query-disk-hash-exist-response"

    def encode(self) -> bytes:
        writer = BytesIO()
        encode_str = self.holder_name.encode(encoding="utf-8")
        writer.write(struct.pack("<h", len(encode_str)))
        writer.write(encode_str)
        writer.write(struct.pack("<H", len(self.states)))
        writer.write(self.states.tobytes())
        return writer.getvalue()


@dataclass
class GetDiskHashPayload:
    hashes: list[HashWithPosition] = field(default_factory=lambda: [])
    packet_name: str = "blob-hash-get-disk-hash-payload"

    def decode(self, bs: bytes | None):
        if bs is None:
            return
        reader = BytesIO(bs)
        self.hashes = [
            decode_hash_with_position(reader)
            for _ in range(struct.unpack("<H", reader.read(2))[0])
        ]


@dataclass
class GetDiskHashPayloadResponse:
    holder_name: str = ""
    payload: list[PayloadByHash] = field(default_factory=lambda: [])
    packet_name: str = "blob-hash-get-disk-hash-payload-response"

    def encode(self) -> bytes:
        writer = BytesIO()
        encode_str = self.holder_name.encode(encoding="utf-8")
        writer.write(struct.pack("<h", len(encode_str)))
        writer.write(encode_str)
        writer.write(struct.pack("<H", len(self.payload)))
        for i in self.payload:
            i.encode(writer)
        return writer.getvalue()


@dataclass
class RequireSyncHashToDisk:
    payload: list[PayloadByHash] = field(default_factory=lambda: [])
    packet_name: str = "blob-hash-require-sync-hash-to-disk"

    def decode(self, bs: bytes | None):
        if bs is None:
            return
        reader = BytesIO(bs)
        for _ in range(struct.unpack("<H", reader.read(2))[0]):
            p = PayloadByHash()
            p.decode(reader)
            self.payload.append(p)



import os
import platform

from ....utils import fmts


def get_bin_path():
    sys_machine = platform.machine().lower()
    sys_type = platform.uname().system

    # Mapping architecture names to common naming
    arch_map = {"x86_64": "amd64", "aarch64": "arm64"}
    sys_machine = arch_map.get(sys_machine, sys_machine)

    # Mapping system types to library file names
    if sys_type == "Windows":
        exe_fn = f"FateArk_windows_{sys_machine}.exe"
    elif "TERMUX_VERSION" in os.environ:
        exe_fn = "FateArk_android_arm64.so"
    elif sys_type == "Linux":
        exe_fn = f"FateArk_linux_{sys_machine}.so"
    else:
        fmts.print_err(f"暂不支持的操作系统: {sys_machine}")
        raise SystemExit
    return os.path.join(os.getcwd(), "tooldelta", "bin", exe_fn)



from enum import IntEnum


class PacketIDs(IntEnum):
    "数据包 ID 常量表"

    # https://prismarinejs.github.io/minecraft-data/protocol/bedrock/1.21.42/#Action

    IDLogin = Login = 1  # 客户端登录（本地）
    IDPlayStatus = PlayStatus = 2  # 玩家状态
    IDServerToClientHandshake = ServerToClientHandshake = 3  # 服务端到客户端握手
    IDClientToServerHandshake = ClientToServerHandshake = 4  # 客户端到服务端握手
    IDDisconnect = Disconnect = 5  # 断开连接
    IDResourcePacksInfo = ResourcePacksInfo = 6  # 资源包信息（非常用）
    IDResourcePackStack = ResourcePackStack = 7  # 资源包堆叠（非常用）
    IDResourcePackClientResponse = ResourcePackClientResponse = (
        8  # 资源包客户端响应（非常用）
    )
    IDText = Text = 9  # 文本消息
    IDSetTime = SetTime = 10  # 更新客户端时间（服务端 -> 客户端）
    IDStartGame = StartGame = 11  # 开始游戏
    IDAddPlayer = AddPlayer = 12  # 添加玩家实体
    IDAddActor = AddActor = 13  # 添加实体
    IDRemoveActor = RemoveActor = 14  # 添加实体
    IDAddItemActor = AddItemActor = 15  # 添加物品实体
    IDTakeItemActor = TakeItemActor = 17  # 捡起物品实体（动画）
    IDMoveActorAbsolute = MoveActorAbsolute = 18  # 移动实体到绝对位置
    IDMovePlayer = MovePlayer = 19  # 玩家移动（服务端 <-> 客户端）
    IDPassengerJump = PassengerJump = 20  # 乘骑跳跃（客户端 -> 服务端）
    IDUpdateBlock = UpdateBlock = 21  # 更新方块（单方块修改）
    IDAddPainting = AddPainting = 22  # 添加绘画实体
    IDTickSync = TickSync = 23  # 同步Tick（服务端 <-> 客户端）
    IDLevelSoundEventV1 = LevelSoundEventV1 = 24  # ???
    IDLevelEvent = LevelEvent = 25  # 世界事件（服务端 -> 客户端）
    IDBlockEvent = BlockEvent = 26  # 方块事件（服务端 -> 客户端）[打开箱子./././...]
    IDActorEvent = ActorEvent = 27  # 实体事件（服务端 -> 客户端）[狼抖干自己./././...]
    IDMobEffect = MobEffect = 28  # 生物效果（服务端 -> 客户端）
    IDUpdateAttributes = UpdateAttributes = (
        29  # 更新实体属性（服务端 -> 客户端）[移动速度./健康状况./...]
    )
    IDInventoryTransaction = InventoryTransaction = 30  # 物品交易（服务端 <- 客户端）
    IDMobEquipment = MobEquipment = (
        31  # 实体物品持有（服务端 <-> 客户端）[僵尸手持石剑././...]
    )
    IDMobArmourEquipment = MobArmourEquipment = (
        32  # 装备穿戴（服务端 -> 客户端）[玩家./僵尸./其他实体./...]
    )
    IDInteract = Interact = 33  # 实体交互（弃用）
    IDBlockPickRequest = BlockPickRequest = 34  # 拾取物品请求（客户端 -> 服务端）
    IDActorPickRequest = ActorPickRequest = 35  # 拾取实体请求（客户端 -> 服务端）
    IDPlayerAction = PlayerAction = 36  # 玩家行为（客户端 -> 服务端）
    IDHurtArmour = HurtArmour = 38  # 盔甲损害（服务端 -> 客户端）
    IDSetActorData = SetActorData = (
        39  # 实体元数据（服务端 -> 客户端）[实体是否着火././...]
    )
    IDSetActorMotion = SetActorMotion = 40  # 设置客户端速度（服务端 -> 客户端）
    IDSetActorLink = SetActorLink = 41  # 设置实体乘骑（服务端 -> 客户端）
    IDSetHealth = SetHealth = 42  # 设置玩家血量（服务端 -> 客户端）
    IDSetSpawnPosition = SetSpawnPosition = 43  # 设置玩家出生点位置（服务端 -> 客户端）
    IDAnimate = Animate = 44  # 动画效果（服务端 -> 客户端）
    IDRespawn = Respawn = 45  # 重生（服务端 <-> 客户端）
    IDContainerOpen = ContainerOpen = 46  # 打开容器（服务端 -> 客户端）
    IDContainerClose = ContainerClose = 47  # 关闭容器（服务端 -> 客户端）
    IDPlayerHotBar = PlayerHotBar = 48  # 玩家快捷栏槽位（服务端 -> 客户端）
    IDInventoryContent = InventoryContent = 49  # 更新玩家背包（服务端 -> 客户端）
    IDInventorySlot = InventorySlot = 50  # 玩家背包单槽位更新（服务端 -> 客户端）
    IDContainerSetData = ContainerSetData = 51  # 容器设置数据（服务端 -> 客户端）
    IDCraftingData = CraftingData = 52  # 合成数据（服务端 -> 客户端）
    IDCraftingEvent = CraftingEvent = 53  # 合成事件（客户端 -> 服务端）
    IDGUIDataPickItem = GUIDataPickItem = 54  # GUI数据拾取物品（客户端 -> 服务端）
    IDAdventureSettings = AdventureSettings = 55  # 冒险设置（服务端 -> 客户端）
    IDBlockActorData = BlockActorData = 56  # 方块实体数据（服务端 -> 客户端）
    IDPlayerInput = PlayerInput = 57  # 玩家输入（客户端 -> 服务端）
    IDLevelChunk = LevelChunk = 58  # 区块数据（服务端 -> 客户端）
    IDSetCommandsEnabled = SetCommandsEnabled = 59  # 设置命令启用（服务端 -> 客户端）
    IDSetDifficulty = SetDifficulty = 60  # 设置难度（服务端 -> 客户端）
    IDChangeDimension = ChangeDimension = 61  # 改变维度（服务端 -> 客户端）
    IDSetPlayerGameType = SetPlayerGameType = 62  # 设置玩家游戏类型（服务端 -> 客户端）
    IDPlayerList = PlayerList = 63  # 玩家列表（服务端 -> 客户端）
    IDSimpleEvent = SimpleEvent = 64  # 简单事件（服务端 -> 客户端）
    IDEvent = Event = 65  # 事件（服务端 -> 客户端）
    IDSpawnExperienceOrb = SpawnExperienceOrb = 66  # 生成经验球（服务端 -> 客户端）
    IDClientBoundMapItemData = ClientBoundMapItemData = (
        67  # 客户端绑定地图物品数据（服务端 -> 客户端）
    )
    IDMapInfoRequest = MapInfoRequest = 68  # 地图信息请求（客户端 -> 服务端）
    IDRequestChunkRadius = RequestChunkRadius = 69  # 请求区块半径（客户端 -> 服务端）
    IDChunkRadiusUpdated = ChunkRadiusUpdated = 70  # 区块半径更新（服务端 -> 客户端）
    IDItemFrameDropItem = ItemFrameDropItem = (
        71  # 物品展示框掉落物品（服务端 -> 客户端）
    )
    IDGameRulesChanged = GameRulesChanged = 72  # 游戏规则改变（服务端 -> 客户端）
    IDCamera = Camera = 73  # 相机（服务端 -> 客户端）
    IDBossEvent = BossEvent = 74  # Boss事件（服务端 -> 客户端）
    IDShowCredits = ShowCredits = 75  # 显示 credits（服务端 -> 客户端）
    IDAvailableCommands = AvailableCommands = 76  # 可用命令（服务端 -> 客户端）
    IDCommandRequest = CommandRequest = 77  # 命令请求（客户端 -> 服务端）
    IDCommandBlockUpdate = CommandBlockUpdate = 78  # 命令块更新（客户端 -> 服务端）
    IDCommandOutput = CommandOutput = 79  # 命令输出（服务端 -> 客户端）
    IDUpdateTrade = UpdateTrade = 80  # 更新交易（服务端 -> 客户端）
    IDUpdateEquip = UpdateEquip = 81  # 更新装备（服务端 -> 客户端）
    IDResourcePackDataInfo = ResourcePackDataInfo = (
        82  # 资源包数据信息（服务端 -> 客户端）
    )
    IDResourcePackChunkData = ResourcePackChunkData = (
        83  # 资源包块数据（服务端 -> 客户端）
    )
    IDResourcePackChunkRequest = ResourcePackChunkRequest = (
        84  # 资源包块请求（客户端 -> 服务端）
    )
    IDTransfer = Transfer = 85  # 传输（服务端 -> 客户端）
    IDPlaySound = PlaySound = 86  # 播放声音（服务端 -> 客户端）
    IDStopSound = StopSound = 87  # 停止声音（服务端 -> 客户端）
    IDSetTitle = SetTitle = 88  # 设置标题（服务端 -> 客户端）
    IDAddBehaviourTree = AddBehaviourTree = 89  # 添加行为树（服务端 -> 客户端）
    IDStructureBlockUpdate = StructureBlockUpdate = (
        90  # 结构方块更新（客户端 -> 服务端）
    )
    IDShowStoreOffer = ShowStoreOffer = 91  # 显示商店优惠（服务端 -> 客户端）
    IDPurchaseReceipt = PurchaseReceipt = 92  # 购买收据（客户端 -> 服务端）
    IDPlayerSkin = PlayerSkin = 93  # 玩家皮肤（客户端 -> 服务端）
    IDSubClientLogin = SubClientLogin = 94  # 子客户端登录（客户端 -> 服务端）
    IDAutomationClientConnect = AutomationClientConnect = (
        95  # 自动化客户端连接（客户端 -> 服务端）
    )
    IDSetLastHurtBy = SetLastHurtBy = 96  # 设置最后受伤来源（服务端 -> 客户端）
    IDBookEdit = BookEdit = 97  # 书本编辑（客户端 -> 服务端）
    IDNPCRequest = NPCRequest = 98  # NPC请求（客户端 -> 服务端）
    IDPhotoTransfer = PhotoTransfer = 99  # 照片传输（客户端 -> 服务端）
    IDModalFormRequest = ModalFormRequest = 100  # 模态表单请求（服务端 -> 客户端）
    IDModalFormResponse = ModalFormResponse = 101  # 模态表单响应（客户端 -> 服务端）
    IDServerSettingsRequest = ServerSettingsRequest = (
        102  # 服务器设置请求（客户端 -> 服务端）
    )
    IDServerSettingsResponse = ServerSettingsResponse = (
        103  # 服务器设置响应（服务端 -> 客户端）
    )
    IDShowProfile = ShowProfile = 104  # 显示资料（客户端 -> 服务端）
    IDSetDefaultGameType = SetDefaultGameType = (
        105  # 设置默认游戏类型（服务端 -> 客户端）
    )
    IDRemoveObjective = RemoveObjective = 106  # 移除目标（服务端 -> 客户端）
    IDSetDisplayObjective = SetDisplayObjective = (
        107  # 设置显示目标（服务端 -> 客户端）
    )
    IDSetScore = SetScore = 108  # 设置分数（服务端 -> 客户端）
    IDLabTable = LabTable = 109  # 实验室表（服务端 -> 客户端）
    IDUpdateBlockSynced = UpdateBlockSynced = 110  # 同步更新方块（服务端 -> 客户端）
    IDMoveActorDelta = MoveActorDelta = 111  # 移动实体增量（服务端 -> 客户端）
    IDSetScoreboardIdentity = SetScoreboardIdentity = (
        112  # 设置计分板身份（服务端 -> 客户端）
    )
    IDSetLocalPlayerAsInitialised = SetLocalPlayerAsInitialised = (
        113  # 设置本地玩家为已初始化（服务端 -> 客户端）
    )
    IDUpdateSoftEnum = UpdateSoftEnum = 114  # 更新软枚举（服务端 -> 客户端）
    IDNetworkStackLatency = NetworkStackLatency = (
        115  # 网络堆栈延迟（服务端 -> 客户端）
    )
    IDScriptCustomEvent = ScriptCustomEvent = 117  # 脚本自定义事件（客户端 -> 服务端）
    IDSpawnParticleEffect = SpawnParticleEffect = (
        118  # 生成粒子效果（服务端 -> 客户端）
    )
    IDAvailableActorIdentifiers = AvailableActorIdentifiers = (
        119  # 可用实体标识符（服务端 -> 客户端）
    )
    IDLevelSoundEventV2 = LevelSoundEventV2 = 120  # 世界声音事件 V2（服务端 -> 客户端）
    IDNetworkChunkPublisherUpdate = NetworkChunkPublisherUpdate = (
        121  # 网络区块发布者更新（服务端 -> 客户端）
    )
    IDBiomeDefinitionList = BiomeDefinitionList = (
        122  # 生物群系定义列表（服务端 -> 客户端）
    )
    IDLevelSoundEvent = LevelSoundEvent = 123  # 世界声音事件（服务端 -> 客户端）
    IDLevelEventGeneric = LevelEventGeneric = 124  # 世界事件通用（服务端 -> 客户端）
    IDLecternUpdate = LecternUpdate = 125  # 讲台更新（客户端 -> 服务端）
    IDAddEntity = AddEntity = 127  # 添加实体（服务端 -> 客户端）
    IDRemoveEntity = RemoveEntity = 128  # 移除实体（服务端 -> 客户端）
    IDClientCacheStatus = ClientCacheStatus = 129  # 客户端缓存状态（客户端 -> 服务端）
    IDOnScreenTextureAnimation = OnScreenTextureAnimation = (
        130  # 屏幕纹理动画（服务端 -> 客户端）
    )
    IDMapCreateLockedCopy = MapCreateLockedCopy = (
        131  # 地图创建锁定副本（服务端 -> 客户端）
    )
    IDStructureTemplateDataRequest = StructureTemplateDataRequest = (
        132  # 结构模板数据请求（客户端 -> 服务端）
    )
    IDStructureTemplateDataResponse = StructureTemplateDataResponse = (
        133  # 结构模板数据响应（服务端 -> 客户端）
    )
    IDClientCacheBlobStatus = ClientCacheBlobStatus = (
        135  # 客户端缓存块状态（客户端 -> 服务端）
    )
    IDClientCacheMissResponse = ClientCacheMissResponse = (
        136  # 客户端缓存未命中响应（服务端 -> 客户端）
    )
    IDEducationSettings = EducationSettings = 137  # 教育设置（服务端 -> 客户端）
    IDEmote = Emote = 138  # 表情（客户端 -> 服务端）
    IDMultiPlayerSettings = MultiPlayerSettings = (
        139  # 多人游戏设置（客户端 -> 服务端）
    )
    IDSettingsCommand = SettingsCommand = 140  # 设置命令（客户端 -> 服务端）
    IDAnvilDamage = AnvilDamage = 141  # 铁砧损坏（服务端 -> 客户端）
    IDCompletedUsingItem = CompletedUsingItem = 142  # 完成使用物品（客户端 -> 服务端）
    IDNetworkSettings = NetworkSettings = 143  # 网络设置（服务端 -> 客户端）
    IDPlayerAuthInput = PlayerAuthInput = 144  # 玩家认证输入（客户端 -> 服务端）
    IDCreativeContent = CreativeContent = 145  # 创造内容（服务端 -> 客户端）
    IDPlayerEnchantOptions = PlayerEnchantOptions = (
        146  # 玩家附魔选项（服务端 -> 客户端）
    )
    IDItemStackRequest = ItemStackRequest = 147  # 物品堆栈请求（客户端 -> 服务端）
    IDItemStackResponse = ItemStackResponse = 148  # 物品堆栈响应（服务端 -> 客户端）
    IDPlayerArmourDamage = PlayerArmourDamage = 149  # 玩家盔甲损坏（服务端 -> 客户端）
    IDCodeBuilder = CodeBuilder = 150  # 代码构建器（客户端 -> 服务端）
    IDUpdatePlayerGameType = UpdatePlayerGameType = (
        151  # 更新玩家游戏类型（服务端 -> 客户端）
    )
    IDEmoteList = EmoteList = 152  # 表情列表（服务端 -> 客户端）
    IDPositionTrackingDBServerBroadcast = (
        153  # 位置跟踪数据库服务器广播（服务端 -> 客户端）
    )
    IDPositionTrackingDBClientRequest = (
        154  # 位置跟踪数据库客户端请求（客户端 -> 服务端）
    )
    IDDebugInfo = DebugInfo = 155  # 调试信息（服务端 -> 客户端）
    IDPacketViolationWarning = PacketViolationWarning = (
        156  # 数据包违规警告（服务端 -> 客户端）
    )
    IDMotionPredictionHints = MotionPredictionHints = (
        157  # 运动预测提示（服务端 -> 客户端）
    )
    IDAnimateEntity = AnimateEntity = 158  # 动画实体（服务端 -> 客户端）
    IDCameraShake = CameraShake = 159  # 相机震动（服务端 -> 客户端）
    IDPlayerFog = PlayerFog = 160  # 玩家迷雾（服务端 -> 客户端）
    IDCorrectPlayerMovePrediction = CorrectPlayerMovePrediction = (
        161  # 纠正玩家移动预测（服务端 -> 客户端）
    )
    IDItemComponent = ItemComponent = 162  # 物品组件（服务端 -> 客户端）
    IDFilterText = FilterText = 163  # 过滤文本（客户端 -> 服务端）
    IDClientBoundDebugRenderer = ClientBoundDebugRenderer = (
        164  # 客户端绑定调试渲染器（服务端 -> 客户端）
    )
    IDSyncActorProperty = SyncActorProperty = 165  # 同步实体属性（服务端 -> 客户端）
    IDAddVolumeEntity = AddVolumeEntity = 166  # 添加体积实体（服务端 -> 客户端）
    IDRemoveVolumeEntity = RemoveVolumeEntity = 167  # 移除体积实体（服务端 -> 客户端）
    IDSimulationType = SimulationType = 168  # 模拟类型（服务端 -> 客户端）
    IDNPCDialogue = NPCDialogue = 169  # NPC对话（服务端 -> 客户端）
    IDEducationResourceURI = EducationResourceURI = (
        170  # 教育资源URI（服务端 -> 客户端）
    )
    IDCreatePhoto = CreatePhoto = 171  # 创建照片（服务端 -> 客户端）
    IDUpdateSubChunkBlocks = UpdateSubChunkBlocks = (
        172  # 更新子区块方块（服务端 -> 客户端）
    )
    IDPhotoInfoRequest = PhotoInfoRequest = 173  # 照片信息请求（客户端 -> 服务端）
    IDSubChunk = SubChunk = 174  # 子区块（服务端 -> 客户端）
    IDSubChunkRequest = SubChunkRequest = 175  # 子区块请求（客户端 -> 服务端）
    IDClientStartItemCooldown = ClientStartItemCooldown = (
        176  # 客户端开始物品冷却（客户端 -> 服务端）
    )
    IDScriptMessage = ScriptMessage = 177  # 脚本消息（客户端 -> 服务端）
    IDCodeBuilderSource = CodeBuilderSource = 178  # 代码构建器源（客户端 -> 服务端）
    IDTickingAreasLoadStatus = TickingAreasLoadStatus = (
        179  #  ticking 区域加载状态（服务端 -> 客户端）
    )
    IDDimensionData = DimensionData = 180  # 维度数据（服务端 -> 客户端）
    IDAgentAction = AgentAction = 181  # 代理
    IDChangeMobProperty = ChangeMobProperty = 182  # 改变生物属性（服务端 -> 客户端）
    IDLessonProgress = LessonProgress = 183  # 课程进度（客户端 -> 服务端）
    IDRequestAbility = RequestAbility = 184  # 请求能力（客户端 -> 服务端）
    IDRequestPermissions = RequestPermissions = 185  # 请求权限（客户端 -> 服务端）
    IDToastRequest = ToastRequest = 186  # 吐司请求（服务端 -> 客户端）
    IDUpdateAbilities = UpdateAbilities = 187  # 更新能力（服务端 -> 客户端）
    IDUpdateAdventureSettings = UpdateAdventureSettings = (
        188  # 更新冒险设置（服务端 -> 客户端）
    )
    IDDeathInfo = DeathInfo = 189  # 死亡信息（服务端 -> 客户端）
    IDEditorNetwork = EditorNetwork = 190  # 编辑器网络（服务端 -> 客户端）
    IDFeatureRegistry = FeatureRegistry = 191  # 特性注册表（服务端 -> 客户端）
    IDServerStats = ServerStats = 192  # 服务器统计（服务端 -> 客户端）
    IDRequestNetworkSettings = RequestNetworkSettings = (
        193  # 请求网络设置（客户端 -> 服务端）
    )
    IDGameTestRequest = GameTestRequest = 194  # 游戏测试请求（客户端 -> 服务端）
    IDGameTestResults = GameTestResults = 195  # 游戏测试结果（服务端 -> 客户端）
    IDUpdateClientInputLocks = UpdateClientInputLocks = (
        196  # 更新客户端输入锁（服务端 -> 客户端）
    )
    IDClientCheatAbility = ClientCheatAbility = (
        197  # 客户端作弊能力（客户端 -> 服务端）
    )
    IDCameraPresets = CameraPresets = 198  # 相机预设（服务端 -> 客户端）
    IDUnlockedRecipes = UnlockedRecipes = 199  # 已解锁配方（服务端 -> 客户端）
    IDPyRpc = PyRpc = 200  # Python远程过程调用（客户端 -> 服务端）
    IDChangeModel = ChangeModel = 201  # 改变模型（服务端 -> 客户端）
    IDStoreBuySucc = StoreBuySucc = 202  # 商店购买成功（服务端 -> 客户端）
    IDNeteaseJson = NeteaseJson = 203  # 网易 JSON（客户端 -> 服务端）
    IDChangeModelTexture = ChangeModelTexture = 204  # 改变模型纹理（服务端 -> 客户端）
    IDChangeModelOffset = ChangeModelOffset = 205  # 改变模型偏移（服务端 -> 客户端）
    IDChangeModelBind = ChangeModelBind = 206  # 改变模型绑定（服务端 -> 客户端）
    IDHungerAttr = HungerAttr = 207  # 饥饿属性（服务端 -> 客户端）
    IDSetDimensionLocalTime = SetDimensionLocalTime = (
        208  # 设置维度本地时间（服务端 -> 客户端）
    )
    IDWithdrawFurnaceXp = WithdrawFurnaceXp = 209  # 提取熔炉经验（客户端 -> 服务端）
    IDSetDimensionLocalWeather = SetDimensionLocalWeather = (
        210  # 设置维度本地天气（服务端 -> 客户端）
    )
    IDCustomV1 = CustomV1 = 223  # 自定义 V1（服务端 -> 客户端）
    IDCombine = Combine = 224  # 组合（服务端 -> 客户端）
    IDVConnection = VConnection = 225  # V 连接（服务端 -> 客户端）
    IDTransport = Transport = 226  # 传输（服务端 -> 客户端）
    IDCustomV2 = CustomV2 = 227  # 自定义 V2（服务端 -> 客户端）
    IDConfirmSkin = ConfirmSkin = 228  # 确认皮肤（服务端 -> 客户端）
    IDTransportNoCompress = TransportNoCompress = 229  # 无压缩传输（服务端 -> 客户端）
    IDMobEffectV2 = MobEffectV2 = 230  # 生物效果 V2（服务端 -> 客户端）
    IDMobBlockActorChanged = MobBlockActorChanged = (
        231  # 生物方块实体改变（服务端 -> 客户端）
    )
    IDChangeActorMotion = ChangeActorMotion = 232  # 改变实体运动（服务端 -> 客户端）
    IDAnimateEmoteEntity = AnimateEmoteEntity = 233  # 动画表情实体（服务端 -> 客户端）
    IDCameraInstruction = CameraInstruction = 300  # 相机指令（服务端 -> 客户端）
    IDCompressedBiomeDefinitionList = CompressedBiomeDefinitionList = (
        301  # 压缩生物群系定义列表（服务端 -> 客户端）
    )
    IDTrimData = TrimData = 302  # 修剪数据（服务端 -> 客户端）
    IDOpenSign = OpenSign = 303  # 打开标志（服务端 -> 客户端）
    IDAgentAnimation = AgentAnimation = 304  # 代理动画（服务端 -> 客户端）


PacketIDS = PacketIDs



import json
import os
from typing import Any
from io import TextIOWrapper

from ..constants import TOOLDELTA_PLUGIN_DATA_DIR
from .safe_writer import safe_write

def safe_json_dump(obj: Any, filepath: str, indent=2) -> None:
    """将一个 json 对象写入一个文件，会自动关闭文件读写接口.

    Args:
        obj (str | dict | list): JSON 对象
        fp (Any): open(...) 打开的文件读写口 或 文件路径
    """
    safe_write(filepath, obj, indent=indent)


def safe_json_load(fp: TextIOWrapper | str) -> Any:
    """从一个文件读取 json 对象，会自动关闭文件读写接口.

    Args:
        fp (TextIOWrapper | str): open(...) 打开的文件读写口 或文件路径

    Returns:
        dict | list: JSON 对象
    """
    if isinstance(fp, str):
        with open(fp, encoding="utf-8") as file:
            return json.load(file)
    with fp as file:
        return json.load(file)


class DataReadError(json.JSONDecodeError):
    """读取数据时发生错误"""


def read_from_plugin(plugin_name: str, file: str, default: dict | None = None) -> Any:
    """从插件数据文件夹读取一个 json 文件，会自动创建文件夹和文件.

    Args:
        plugin_name (str): 插件名
        file (str): 文件名
        default (dict, optional): 默认值，若文件不存在则会写入这个默认值

    Raises:
        DataReadError: 读取数据时发生错误
        err: 读取文件路径时发生错误

    Returns:
        dict | list: JSON 对象
    """
    if file.endswith(".json"):
        file = file[:-5]
    filepath = os.path.join(TOOLDELTA_PLUGIN_DATA_DIR, plugin_name, f"{file}.json")
    os.makedirs(os.path.join(TOOLDELTA_PLUGIN_DATA_DIR, plugin_name), exist_ok=True)
    try:
        if default is not None and not os.path.isfile(filepath):
            safe_json_dump(default, filepath)
            return default
        with open(filepath, encoding="utf-8") as f:
            res = safe_json_load(f)
        return res
    except json.JSONDecodeError as err:
        # 判断是否有 msg.doc.pos 属性
        raise DataReadError(err.msg, err.doc, err.pos)


def write_to_plugin(plugin_name: str, file: str, obj: Any, indent=4) -> None:
    """将一个 json 对象写入插件数据文件夹，会自动创建文件夹和文件.

    Args:
        plugin_name (str): 插件名
        file (str): 文件名
        obj (str | dict[Any, Any] | list[Any]): JSON 对象
    """
    os.makedirs(f"{TOOLDELTA_PLUGIN_DATA_DIR}/{plugin_name}", exist_ok=True)
    safe_json_dump(obj, f"{TOOLDELTA_PLUGIN_DATA_DIR}/{plugin_name}/{file}.json", indent=indent)
